@@ -32,7 +32,7 @@ Version 0.01
 
 =cut
 
-our $VERSION = '2.45';
+our $VERSION = '2.46';
 
 =head1 SYNOPSIS
 
@@ -228,6 +228,7 @@ has 'nodelist' => (
     is => 'rw',
     isa => 'ArrayRef',
     required => 0,
+    default => sub { return [] },
     documentation => q{List of nodes to submit jobs to. Defaults to the partition with the most nodes.},
 );
 
@@ -285,9 +286,9 @@ has 'template_file' => (
 [% END %]
 
 [% IF MODULE %]
-    [% FOR d = MODULE %]
+[% FOR d = MODULE %]
 module load [% d %]
-    [% END %]
+[% END %]
 [% END %]
 
 [% COMMAND %]
@@ -564,6 +565,18 @@ has 'slurmfile' => (
     #default => 0,
 #);
 
+=head2 slurm_decides
+
+Do not specify a node or partition in your sbatch file. Let Slurm decide which nodes/partition to submit jobs.
+
+=cut
+
+has 'slurm_decides' => (
+    is => 'rw',
+    isa => 'Bool',
+    default => 0,
+);
+
 =head1 SUBROUTINES/METHODS
 
 =cut
@@ -629,6 +642,7 @@ sub get_nodes{
 
     $DB::single=2;
 #    #Fix - had this backwards
+    return if $self->slurm_decides;
     if($self->nodelist && !$self->partition){
         print "If you define a nodelist you must define a partition!\n";
         die;
@@ -747,74 +761,80 @@ sub parse_file_slurm{
         my $line = $_;
         next unless $line;
         next unless $line =~ m/\S/;
-        if($line =~ m/^#HPC/){
-            $self->check_meta($line);
-        }
-        next if $line =~ m/^#/;
-
-        $DB::single=2;
-
-        #Do a sanity check for nohup
-        if($line =~ m/^nohup/){
-            die print "You cannot submit jobs to the queue using nohup! Please remove nohup and try again.\n";
-        }
-
-        $DB::single=2;
-        #if( $self->cmd_counter > 0 && 0 == $self->cmd_counter % ($self->commands_per_node + 1) && $self->batch ){
-        if( $self->cmd_counter > 0 && 0 == $self->cmd_counter % ($self->commands_per_node) && $self->batch ){
-            #Run this batch and start the next
-            $DB::single=2;
-            $self->work;
-            push(@{$self->jobref}, []) if $self->serial;
-        }
-
-        if($self->has_cmd){
-            $self->add_cmd($line);
-            $self->add_batch($line);
-            if($line =~ m/\\$/){
-                $DB::single=2;
-                next;
-            }
-            else{
-                $self->add_cmd("\n");
-                $self->add_batch("\n");
-                $self->clear_cmd;
-                $self->inc_cmd_counter;
-            }
-        }
-        else{
-            $self->add_cmd($line);
-
-            if($line =~ m/\\$/){
-                $self->add_batch($line);
-                next;
-            }
-            elsif( $self->match_cmd(qr/^wait$/) ){
-                #submit this batch and get the job id so the next can depend upon it
-                $DB::single=2;
-                $self->clear_cmd;
-                $self->wait(1);
-                $self->work if $self->has_batch;
-                push(@{$self->jobref}, []);
-            }
-            elsif( $self->match_cmd(qr/^newnode$/) ){
-                $self->clear_cmd;
-                $self->work if $self->has_batch;
-                push(@{$self->jobref}, []) if $self->serial;
-            }
-            else{
-                #Don't want to increase command count for wait and newnode
-                $self->inc_cmd_counter;
-            }
-            $self->add_batch($line."\n") if $self->has_cmd;
-            $DB::single=2;
-            $self->clear_cmd;
-        }
+        $self->process_lines($line);
     }
     $self->work if $self->has_batch;
     push(@{$self->jobref}, []) if $self->serial;
 }
 
+sub process_lines {
+    my $self = shift;
+    my $line = shift;
+
+    $self->check_meta($line);
+    return if $line =~ m/^#/;
+
+    $DB::single=2;
+
+    #Do a sanity check for nohup
+    if($line =~ m/^nohup/){
+        die print "You cannot submit jobs to the queue using nohup! Please remove nohup and try again.\n";
+    }
+
+    $DB::single=2;
+    #if( $self->cmd_counter > 0 && 0 == $self->cmd_counter % ($self->commands_per_node + 1) && $self->batch ){
+    if( $self->cmd_counter > 0 && 0 == $self->cmd_counter % ($self->commands_per_node) && $self->batch ){
+        #Run this batch and start the next
+        $DB::single=2;
+        $self->work;
+        push(@{$self->jobref}, []) if $self->serial;
+    }
+
+    if($self->has_cmd){
+        $self->add_cmd($line);
+        $self->add_batch($line);
+        if($line =~ m/\\$/){
+            $DB::single=2;
+            next;
+        }
+        else{
+            $self->add_cmd("\n");
+            $self->add_batch("\n");
+            $self->clear_cmd;
+            $self->inc_cmd_counter;
+        }
+    }
+    else{
+        $self->add_cmd($line);
+
+        if($line =~ m/\\$/){
+            $self->add_batch($line);
+            next;
+        }
+        elsif( $self->match_cmd(qr/^wait$/) ){
+            #submit this batch and get the job id so the next can depend upon it
+            $DB::single=2;
+            $self->clear_cmd;
+            $self->wait(1);
+            $self->work if $self->has_batch;
+            push(@{$self->jobref}, []);
+        }
+        elsif( $self->match_cmd(qr/^newnode$/) ){
+            $self->clear_cmd;
+            $self->work if $self->has_batch;
+            push(@{$self->jobref}, []) if $self->serial;
+        }
+        else{
+            #Don't want to increase command count for wait and newnode
+            $self->inc_cmd_counter;
+        }
+        $self->add_batch($line."\n") if $self->has_cmd;
+        $DB::single=2;
+        $self->clear_cmd;
+    }
+
+}
+
 =head2 check_meta
 
 allow for changing parameters mid through the script
@@ -835,10 +855,17 @@ sub check_meta{
     my $line = shift;
     my(@match, $t1, $t2);
 
+    return unless $line =~ m/^#HPC/;
+
     @match = $line =~ m/HPC (\w+)=(.+)$/;
     ($t1, $t2)  = ($match[0], $match[1]);
 
     $DB::single=2;
+    if(!$self->can($t1)){
+        print "Option $t1 is an invalid option!\n";
+        return;
+    }
+
     if($t1){
         if($t1 eq "module"){
             $self->$t1([$t2]);
@@ -939,6 +966,7 @@ splitting this off from the main command
 =cut
 
 #TODO add support for custom commands
+#TODO Change this all to a plugin system
 
 sub process_batch_command {
     my($self) = @_;
@@ -954,7 +982,7 @@ sub process_batch_command {
     }
     elsif($self->use_gnuparallel){
         $command = "cd ".getcwd()."\n";
-        $command .= "cat ".$self->cmdfile." | parallelparser.pl | parallel --joblog ".$self->outdir."/main.log --gnu -N 1 -q  gnuparallelrunner.pl --command `echo {}` --outdir ".$self->outdir." --logname ".$self->jobname."_".$self->batch_counter."\n";
+        $command .= "cat ".$self->cmdfile." | parallelparser.pl | parallel --joblog ".$self->outdir."/main.log --gnu -N 1 -q  gnuparallelrunner.pl --command `echo {}` --outdir ".$self->outdir." --logname ".$self->jobname."_".$self->batch_counter." --seq {#}"."\n";
     }
     elsif($self->use_threads){
         $command = "cd ".getcwd()."\n"; #Go back to the working directory just to be safe
@@ -1089,11 +1117,20 @@ L<http://search.cpan.org/dist/HPC-Runner-Slurm/>
 
 =head1 Acknowledgements
 
+Before Version 2.41
+
 This module was originally developed at and for Weill Cornell Medical
 College in Qatar within ITS Advanced Computing Team. With approval from
 WCMC-Q, this information was generalized and put on github, for which
 the authors would like to express their gratitude.
 
+As of Version 2.41:
+
+This modules continuing development is supported by NYU Abu Dhabi in the Center for Genomics and Systems Biology.
+With approval from NYUAD, this information was generalized and put on bitbucket, for which
+the authors would like to express their gratitude.
+
+
 =head1 LICENSE AND COPYRIGHT
 
 Copyright 2014 Weill Cornell Medical College in Qatar.
